@@ -18,14 +18,14 @@ __version__ = "0.0.17"
 POWERFLOW_STATUS_TEXT = {
     -1: "Outward",
 }
-
+DEFAULT_UA = "PVMaster/2.0.4 (iPhone; iOS 11.4.1; Scale/2.00)"
 API_URL = "https://semsportal.com/api/"
 
 
 class API():
     """ API implementation """
 
-    #pylint: disable=too-many-instance-attributes
+    #pylint: disable=too-many-instance-attributes,too-many-arguments
     def __init__(
         self,
         system_id: str,
@@ -33,16 +33,16 @@ class API():
         password: str,
         api_url: str=API_URL,
         log_level: Optional[str]=None,
-        user_agent: str="PVMaster/2.0.4 (iPhone; iOS 11.4.1; Scale/2.00)",
+        user_agent: str=DEFAULT_UA,
         skipload: bool=False,
     ) -> None:
         """
         Options:
 
-        #TODO: lang: Real Soon Now it'll filter out any responses without that language
         skipload: don't run self.getCurrentReadings() on init
         api_url: you can change the API endpoint it hits
         """
+        #TODO: lang: Real Soon Now it'll filter out any responses without that language
 
         if log_level is None:
             if "LOG_LEVEL" in os.environ:
@@ -93,7 +93,7 @@ class API():
 
         # GOODWE server
         self.data = self.call(
-            "v1/PowerStation/GetMonitorDetailByPowerstationId", payload
+            "v2/PowerStation/GetMonitorDetailByPowerstationId", payload
         )
 
         retval = self.data
@@ -104,14 +104,14 @@ class API():
                     "no inverter data, try %s, trying again in %s seconds", retry, delay
                 )
                 time.sleep(delay)
-                retval = self.getCurrentReadings(
+                return self.getCurrentReadings(
                     raw=raw, retry=retry + 1, maxretries=maxretries, delay=delay
                 )
-            else:
-                logging.error("No inverter data after %s retries, quitting.", retry)
-                sys.exit(f"No inverter data after {retry} retries, quitting.")
+            logging.error("No inverter data after %s retries, quitting.", retry)
+            sys.exit(f"No inverter data after {retry} retries, quitting.")
         return retval
 
+    # stub function names to old names
     getCurrentReadings = get_current_readings
 
     # def getDayReadings(self, date):
@@ -119,7 +119,7 @@ class API():
     #     payload = {
     #         'powerStationId' : self.system_id
     #     }
-    #     data = self.call("v1/PowerStation/GetMonitorDetailByPowerstationId", payload)
+    #     data = self.call("v2/PowerStation/GetMonitorDetailByPowerstationId", payload)
     #     if 'info' not in data:
     #     logging.warning(date_s + " - Received bad data " + str(data))
     #         return result
@@ -194,10 +194,10 @@ class API():
             filename = f"Plant_Power_{datestr}.xls"
         logging.debug("Will write data for %s to file: %s", datestr, filename)
 
-        # {"api":"v1/PowerStation/ExportPowerstationPac","param":{"date":"2021-12-20","pw_id":"<my-pw-id>"
+        # {"api":"v2/PowerStation/ExportPowerstationPac","param":{"date":"2021-12-20","pw_id":"<my-pw-id>"
         payload = {
             "str" : json.dumps({
-                "api": "v1/PowerStation/ExportPowerstationPac",
+                "api": "v2/PowerStation/ExportPowerstationPac",
                 "param": {
                     "date": datestr,
                     "pw_id": self.system_id,
@@ -216,7 +216,7 @@ class API():
         # full_url = f"{fixed_api_endpoint}"
         response = requests.post(
             url=fixed_api_endpoint,  # full_url,
-            params={"s": "v1/PowerStation/ExportPowerstationPac"},
+            params={"s": "v2/PowerStation/ExportPowerstationPac"},
             headers=self.headers,
             data=payload,
             timeout=timeout,
@@ -251,7 +251,7 @@ class API():
         }
 
         file_data = self.call(
-            "v1/ReportData/GetStationPowerDataFilePath",
+            "v2/ReportData/GetStationPowerDataFilePath",
             download_payload,
             timeout=timeout,
         )
@@ -288,7 +288,7 @@ class API():
         }
         try:
             response = self.session.post(
-                self.global_url + "v1/Common/CrossLogin",
+                self.global_url + "v2/Common/CrossLogin",
                 headers=self.headers,
                 data=login_payload,
                 timeout=timeout,
@@ -458,7 +458,7 @@ class API():
 
 class SingleInverter(API):
     """ API implementation for an account with a single inverter """
-
+    # pylint: disable=too-many-arguments
     def __init__(
         self,
         system_id: str,
@@ -466,7 +466,7 @@ class SingleInverter(API):
         password: str,
         api_url: str=API_URL,
         log_level: Optional[str]=None,
-        user_agent: str="PVMaster/2.0.4 (iPhone; iOS 11.4.1; Scale/2.00)",
+        user_agent: str=DEFAULT_UA,
         skipload: bool=False,
         ) -> None:
         self.loadflow = 0.0
@@ -494,7 +494,7 @@ class SingleInverter(API):
         # update the data
         super().get_current_readings(raw)
         # reduce self.data['inverter'] to a single dict from a list
-        retval = False
+        retval = None
         if self.data.get("inverter"):
             self.data["inverter"] = self.data["inverter"][0]
         else:
@@ -503,12 +503,11 @@ class SingleInverter(API):
                     "no inverter data, try %s, trying again in %s seconds", retry, delay
                 )
                 time.sleep(delay)
-                retval = self.get_current_readings(
+                return self.get_current_readings(
                     raw=raw, retry=retry + 1, maxretries=maxretries, delay=delay
                 )
-            else:
-                logging.error("No inverter data after %s retries, quitting.", retry)
-                sys.exit(f"No inverter data after {retry} retries, quitting.")
+            logging.error("No inverter data after %s retries, quitting.", retry)
+            sys.exit(f"No inverter data after {retry} retries, quitting.")
         return retval
     getCurrentReadings = get_current_readings
 
@@ -603,3 +602,29 @@ class SingleInverter(API):
         if not self.data:
             self.get_current_readings(True)
         return float(self.data["inverter"]["tempperature"])
+
+    def getDataPvoutput(
+        self
+        ) -> Dict[str, Union[str, float]]:  # pylint: disable=invalid-name
+        """updates and returns the data necessary for a one-shot pvoutput upload
+        'd' : testdate.strftime("%Y%m%d"),
+        't' : testtime.strftime("%H:%M"),
+        'v2' : 500, # power generation
+        'v4' : 450,
+        'v5' : 23.5, # temperature
+        'v6' : 234.0, # voltage
+        """
+        if not self.data:
+            self.getCurrentReadings()
+        # "time": "10/04/2019 14:37:29"
+        timestamp = datetime.strptime(
+            self.data.get("info", {}).get("time"), "%m/%d/%Y %H:%M:%S"
+        )
+        data: Dict[str, Union[str, float]] = {}
+        data["d"] = timestamp.strftime("%Y%m%d")  # date
+        data["t"] = timestamp.strftime("%H:%M")  # time
+        data["v2"] = self.getPVFlow()  # PV Generation
+        data["v4"] = self.getLoadFlow()  # power consumption
+        data["v5"] = self.get_inverter_temperature()  # inverter temperature
+        data["v6"] = self.getVoltage()  # voltage
+        return data
